function [out] = casadi_struct(s,varargin)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%@Course      : Master Thesis Spring 2018 
%@Task        : Function for creating a struct from a set of input
%               arguments
%@input       : input arguments (varargin), struct (s) 
%            
%@output      : struct (out) 

%@author      : Joris Gillis, Rien Quirynen, Joel Andersson,
%               Sebastien Gros and Moritz Diehl
%@modified    : Adriaen Verheyleweghen
%@organization: Faculty of Engineering, University of Frieburg, Germany
%@requires    : MATLAB R2016a (not tested in other releases)     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  import casadi.*
  
  out = struct;
  
  origs = s;
  
  if ischar(varargin{1})
      default = 0;
  else
      default = varargin{1};
      varargin = varargin(2:end);
  end
  
  tmp = {default,varargin{:}};
  
  subs = {};
  
  for i = fliplr(1:length(tmp)/2)
      c = varargin{2*i};
      if isa(c,'cell')
          try 
              subs = {subs{:},varargin{2*i-1},s.(varargin{2*i-1}),c};
              s = rmfield(s,varargin{2*i-1});
              tmp = {tmp{1:2*i-1},tmp{2*(i+1):end}};
          catch err
              rethrow(err)
          end
      end
  end
  
  for j = 1:length(subs)/3
    arg = subs{3*j};
    out.(subs{3*j-2}) = casadi_struct(subs{3*j-1},arg{:});
  end
  
  for k=fieldnames(s)'
    k = k{1};
    found = -1;
    for l=1:length(varargin)/2
        if strcmp(varargin{2*l-1},k)
           found = l;
           break;
        end
    end
    if found>0
      if isa(s.(k),'struct')
          e = casadi_struct(s.(k),varargin{2*found:end});
      else
          e = varargin{2*found};
      end
      if isscalar(e)
        dims = size(s.(k));
        e = repmat(e,dims(1),dims(2));
      end
      dims = size(s.(k));
      assert(size(e,1)==dims(1))
      assert(size(e,2)==dims(2))
    else
      if isa(s.(k),'struct')
          e = casadi_struct(s.(k),tmp{1:end});
      else
          dims = size(s.(k));
          e = default*DM.ones(dims(1),dims(2));
      end
    end
    out.(k) = e;
  end
  out = orderfields(out,origs);
end